@@ -65,7 +65,7 @@ with other DBs.
 use v5.12;
 
 package SQL::Steno;
-our $VERSION = 'v0.2.1';	# quotes because of CPAN indexer bug
+our $VERSION = 'v0.2.2';	# quotes because of CPAN indexer bug
 
 use strict;
 use Time::HiRes qw(gettimeofday tv_interval);
@@ -131,10 +131,10 @@ Queries
 
 
     [ps => '   show processlist (without Sleep)',
-      '/^(?!.+\|Sleep)/=show processlist'],
+      '{($_[7] // "") ne "Sleep"}=show processlist'],
 
     [psf => '   show full processlist (without Sleep)',
-      '/^(?!.+\|Sleep)/=show full processlist'],
+      '{($_[7] // "") ne "Sleep"}=show full processlist'],
 
 
     [s => 'var,value   set @var = value',
@@ -509,6 +509,7 @@ sub convert {
 	    s/^(?!se(?:lec)?t)/select /i;
     }
 
+    s/ $//mg;
     s/ {2,}/ /g;
     s/\cA(\d+)\cB/$strings[$1]/g; # put back the strings
 
@@ -548,58 +549,41 @@ my %esc = map { $_ eq 'v' ? "\013" : eval( qq!"\\$_"! ), "\\$_" } qw'0 a b e f n
 	}
 	1;
     }
-
-    my $esc_re = join '', sort keys %esc;
-    $esc_re =~ s!\\!\\\\!;
-    sub yaml_quote(;$) {
-	if( !defined ) {
-	    $_ = '~';
-	} elsif( /^(?:y(?:es)?|no?|true|false|o(?:n|ff)|-?\.inf|\.nan)$/s ) { # can only be string in Perl or DB
-	    $_ = "'$_'";
-	} elsif( tr/][{},?:`'"|<>&*!%#@=~\0-\010\013-\037\177-\237-// or $_[0] && tr/\n// ) {
-	    s/([$esc_re])/$esc{$1}/go;
-	    s/([\0-\010\013-\037\177-\237])/sprintf "\\x%02x", ord $1/ge;
-	    $_ = qq!"$_"!;
-	} elsif( tr/\n//d ) {
-	    my $nl = chomp;
-	    s/^/    /mg;
-	    substr $_, 0, 0, $nl ? "|2\n" : "|2-\n";
-	}
-    }
 }
 
-sub render_csv($;$$) {
-    my( $sth, $re, $sub ) = @_;
-    my $filter;			# don't filter names
+sub render_csv($;$) {
+    my( $sth, $filter ) = @_;
     my $name = $sth->{NAME};
     my @row = @$name;
-    while( @row ) {
-	next if $sub && $filter && !$sub->( \@row, $name );
-	next if $re  && $filter && join( '|', map $_ // '\@', @row ) !~ $re;
+    while() {
 	for( @row ) {
-	    $_ //= '';
-	    $_ = qq!"$_"! if
-		s/"/""/g or
-		tr/,\n// or
-		/\A=/;
+	    if( defined ) {
+		$_ = qq!"$_"! if
+		    /\A\Z/ or
+		    s/"/""/g or
+		    tr/,\n// or
+		    /\A=/;
+	    } else {
+		$_ = '';
+	    }
 	    utf8::decode $_;
 	}
 	print join( ',', @row ) . "\n";
-    } continue {
-	@row = $sth->fetchrow_array;
-	$filter = 1;
+
+      FETCH:
+	@row = $sth->fetchrow_array
+	    or last;
+	$filter->( $name, @row ) or goto FETCH if $filter;
     }
 }
 
-sub render_tabular($;$$) {
-    my( $sth, $re, $sub ) = @_;
-    my $filter;			# don't filter names
+sub render_tabular($;$) {
+    my( $sth, $filter ) = @_;
     my @name = @{$sth->{NAME}};
     my @len = (0) x @name;
     my( @txt, @res, @comp );
     while( my @res1 = $sth->fetchrow_array ) {
-	next if $sub && $filter && !$sub->( \@res1, \@name );
-	next if $re  && $filter && join( '|', map $_ // '\@', @res1 ) !~ $re;
+	next if $filter && !$filter->( \@name, @res1 );
 	for my $i ( 0..$#res1 ) {
 	    if( !defined $res1[$i] ) {
 		$res1[$i] = '\@';
@@ -625,8 +609,6 @@ sub render_tabular($;$$) {
 	    @comp = @res1;
 	}
 	push @res, \@res1;
-    } continue {
-	$filter = 1;
     }
     if( @res ) {
 	@comp = () if @res == 1;
@@ -665,36 +647,55 @@ sub render_tabular($;$$) {
     }
 }
 
-sub render_yaml($;$$) {
-    my( $sth, $re, $sub ) = @_;
-    my @label = my @name = @{$sth->{NAME}};
-    my $maxlen = 0;
-    for( @label ) {
-	yaml_quote 1;
-	substr $_, 0, 0, $maxlen ? '  ' : '- ';
-	$maxlen = length if $maxlen < length;
-    }
-    $_ .= ': ' . ' ' x ($maxlen - length)
-	for @label;
-    while( my @res = $sth->fetchrow_array ) {
-	next if $sub && !$sub->( \@res, \@name );
-	next if $re  && join( '|', map $_ // '\@', @res ) !~ $re;
-	my $count = count @res;		# init
-	my $i;
+my $yaml_re = join '', sort keys %esc;
+$yaml_re =~ s!\\!\\\\!;
+sub render_yaml($;$) {
+    my( $sth, $filter ) = @_;
+    my @label;			       # Fill on 0th round with same transformation as data (but \n inline)
+    my $count = count $DBI::rows || 1; # init \todo don't know how many unfiltered
+    my @row = @{$sth->{NAME}};
+    while() {
 	local $_;
-	while( ($i, $_) = each @res ) {
-	    yaml_quote;
-	    printf "$label[$i]$_\n";
-	    defined count or last if defined $count;
+	my $i = 0;
+	for( @row ) {
+	    if( !defined ) {
+		$_ = '~';
+	    } elsif( /^(?:y(?:es)?|no?|true|false|o(?:n|ff)|-?\.inf|\.nan)$/s ) { # can only be string in Perl or DB
+		$_ = "'$_'";
+	    } elsif( tr/][{},?:`'"|<>&*!%#@=~\0-\010\013-\037\177-\237-// or @label ? 0 : tr/\n// ) {
+		s/([$yaml_re])/$esc{$1}/go;
+		s/([\0-\010\013-\037\177-\237])/sprintf "\\x%02x", ord $1/ge;
+		$_ = qq!"$_"!;
+	    } elsif( tr/\n//d ) {
+		my $nl = chomp;
+		s/^/    /mg;
+		substr $_, 0, 0, $nl ? "|2\n" : "|2-\n";
+	    }
+	    printf "$label[$i++]$_\n" if @label;
 	}
+	if( @label ) {
+	    defined count or last if defined $count;
+	} else {
+	    my $maxlen = 0;
+	    for( @row ) {
+		substr $_, 0, 0, $maxlen ? '  ' : '- ';
+		$maxlen = length if $maxlen < length;
+	    }
+	    $_ .= ': ' . ' ' x ($maxlen - length)
+		for @label = @row;
+	}
+      FETCH:
+	@row = $sth->fetchrow_array
+	    or last;
+	$filter->( $sth->{NAME}, @row ) or goto FETCH if $filter;
     }
 }
 
 
 
 my $lasttime = time;
-sub run($;$$) {
-    my( $sql, $re, $sub ) = @_;
+sub run($;$) {
+    my( $sql, $filter ) = @_;
     my $t0 = [gettimeofday];
     if( $DBI::err || $t0->[0] - $lasttime > 3600 and !$dbh->ping ) {
 	printf STDOUT "Inactive for %ds, ping failed after %.03fs, your session variables are lost.\n",
@@ -711,9 +712,9 @@ sub run($;$$) {
 	    tv_interval( $t0, $t1 ), tv_interval( $t1 ), $DBI::rows;
 	if( $sth->{Active} ) {
 	    if( $render ) {
-		&$render( $sth, $re, $sub );
+		&$render( $sth, $filter );
 	    } else {
-		render_tabular $sth, $re, $sub;
+		render_tabular $sth, $filter;
 	    }
 	}
     }
@@ -749,19 +750,38 @@ handle run that instead of this statement.  Else replace with what it returns.
 
 Reprocess result as a shell entry (i.e. it may return another C<&>query).
 
+=item /regexp/ statement
+
+=item /regexp/i statement
+
+=item !/regexp/ statement
+
+=item !/regexp/i statement
+
+This will treat the I<statement> normally, but will join each output row for
+matching as though it were the only row in default tabular format.  That is
+there is no space padding and the character '|' (which must be matched as
+C<\|>) separates columns.  At this stage no value rewriting has occured,
+except NULL is already rendered as C<\@>.
+
+With a suffix C<i>, matching becomes case insensitive.  This is why the mostly
+optional space is shown above.  Without an C<i>, but if the statement starts
+with the word C<i> (e.g. your first column name), you must separate it with a
+space.  With an C<i>, if the statement starts with an alphanumeric caracter,
+you must separate it with a space.
+
+Only matching rows are considered unless there is a preceding C<!>, in which
+case only non-matching rows are considered.
+
 =item {Perl code}statement
 
 Call I<Perl code> for every output row returned by the I<statement> with the
-array of values as first and the array of column names as second argument.  It
-may modify individual values, but not the array.  If it returns false, the row
-is skipped.
+array of column names as zeroth argument and the values after that (i.e.
+numbered from 1 like in SQL).  It may modify individual values.  If it returns
+false, the row is skipped.
 
-=item /regexp/statement
-
-This will treat the I<statement> normally, but will join each output row with
-'|' characters (which need to be matched as C<\/>) and check that against the
-regexp.  Only matching rows are considered.  You may combine S<C<{Perl
-code}/regexp/>>.
+You may combine S<C</regexp/{Perl code}>> in any order and as many of them as
+you want.
 
 =item =literal sql
 
@@ -819,7 +839,6 @@ sub shell() {
 	    s/\A\s+//;
 	}
 	s/\s+\Z//;
-
 	until( $error ) {
 	    last unless
 		s!^\&(\.?\w+|-)(\(((?:(?>[^()]+)|(?2))*)\))!convert_Queries $1, $3!e
@@ -840,13 +859,18 @@ sub shell() {
 	    }
 	}
 
-	my $sub;
-	if( s/^\s*$perl_re// ) {
-	    $sub = eval "sub $1";
+	my $filter = '';
+	while( s/^\s*$perl_re// || s%^\s*(!?)(/.+?/(?:i\b)?)\s*%% ) {
+	    if( defined $2 ) {
+		$filter .= 'return if join( "|", map ref() ? () : $_ // q!\@!, @_ ) ' . ($1 ? '=' : '!') .  "~ $2;\n";
+	    } else {
+		$filter .= "return unless eval $1;\n";
+	    }
+	}
+	if( $filter ) {
+	    $filter = eval "sub {\n$filter 1; }";
 	    warn $@ if $@;
 	}
-	my $re = $1 ? qr/$2/i : qr/$2/
-	    if s!^(i?)/(.+?)/\s*!!;
 	goto RUN if s/^\s*=//;	# run literally
 
 	my $skip = 0;
@@ -867,7 +891,7 @@ sub shell() {
 		printf STDERR "child died with signal %d, with%s coredump\n",
 		    $exit, ($? & 0b1000_0000) ? '' : 'out';
 	    } else {
-		printf STDERR "child exited with value %d %d\n", $?, $? >> 8;
+		printf STDERR "child exited with value %d\n", $? >> 8;
 	    }
 	    goto NEXT;
 	}
@@ -891,7 +915,7 @@ sub shell() {
 	goto NEXT if $skip;
 
       RUN:
-	run $_, $re, $sub if $dbh;
+	run $_, $filter if $dbh;
 	undef $render;
 	if( $fh ) {
 	    close;
